# Useful links

## Strategic

- The Government Analysis Function's [RAP strategy](https://analysisfunction.civilservice.gov.uk/policy-store/reproducible-analytical-pipelines-strategy/) details the roles we all have to play in working towards RAP.
    - The ONS have also published their [RAP Strategy Implementation Plan 2023](https://www.ons.gov.uk/aboutus/whatwedo/programmesandprojects/analysisfunctionrapstrategy2023implementationplan)
- The [Goldacre Review](https://www.gov.uk/government/publications/better-broader-safer-using-health-data-for-research-and-analysis) mentions RAP extensively as a key tool in improving the use of patient data.
- The Bennett Institute of Applied Data Science has written a [helpful summary](https://www.bennett.ox.ac.uk/blog/2022/07/bennett-insights-an-overview-of-uk-data-policy-developments/) that is helpful for understanding the above documents in the context of UK data policy developments.

## Guidance and Standards

- The [AQUA book of analytical standards](https://www.gov.uk/government/publications/the-aqua-book-guidance-on-producing-quality-analysis-for-government).
- The [ONS best practice team have a useful website - often called the Quack book](https://best-practice-and-impact.github.io/qa-of-code-guidance/intro.html) covering many of the same topics we cover here. Their [best-practice checklist](https://best-practice-and-impact.github.io/qa-of-code-guidance/checklist_higher.html) is particularly useful.
- The Turing Institute's [The Turing Way handbook to reproducible, ethical and collaborative data science](https://the-turing-way.netlify.app/welcome.html)
- We have taken inspiration from the [NHS Digital software engineering COP](https://github.com/NHSDigital/software-engineering-quality-framework/blob/master/insights/review.md).
- [**NHS PyCom Coding Club Github**][1]: Lots of great guides and lessons here from the NHS Python Community

## Examples, Challenges, Benefits

- [This blog post](https://dataingovernment.blog.gov.uk/2017/03/27/reproducible-analytical-pipeline/), from 2017, explains the context and history of RAP in government statistics.
- The GSS's list of [benefits that come from RAP](https://gss.civilservice.gov.uk/reproducible-analytical-pipelines/benefits-to-government-from-reproducible-analytical-pipelines/).
- The Statistics Authority's excellent [report on overcoming barriers to RAP adoption](https://osr.statisticsauthority.gov.uk/publication/reproducible-analytical-pipelines-overcoming-barriers-to-adoption/).
- Alston, J. M., and Rick, J. A.. 2020. [A Beginner's Guide to Conducting Reproducible Research](https://doi.org/10.1002/bes2.1801). Bull Ecol Soc Am 102(2):e01801.
- The GSS training portal offers an [Introduction to RAP course](https://gss.civilservice.gov.uk/training/introduction-to-reproducible-analytical-pipelines-rap/)
- The Government Analysis Function conducted [research on the rollout of RAP across different departments](https://best-practice-and-impact.github.io/CARS-3/index.html).

## Community spaces

- There are several communities to discuss Python, R and Git in the Health and 
  Public sectors:
    - **Government Data Science community** ([website](https://www.gov.uk/service-manual/communities/data-science-community) | 
      [slack](https://govdatascience.slack.com/) #rap_collaboration | [GitHub](https://github.com/ukgovdatascience))
        - **Government Analysis Function RAP Champion network** ([website](https://analysisfunction.civilservice.gov.uk/support/reproducible-analytical-pipelines/reproducible-analytical-pipeline-rap-champions/)) 
    - **NHS Python Community (NHS-pycom)** ([website](https://nhs-pycom.net/) | [slack](https://nhs-pycom.slack.com) | 
      [GitHub](https://github.com/nhs-pycom) | [Coding Club][1])
    - **NHS R Community** ([website](https://nhsrcommunity.com/) | [slack](https://nhsrcommunity.slack.com) | 
      [GitHub](https://github.com/nhs-r-community))
    - **AnalystX** ([website](https://analystx.uk/) | [Future NHS - AnalystX](https://future.nhs.uk/connect.ti/DataAnalytics/grouphome)
      | [GitHub](https://github.com/nhs-analystx))
- We have an **NHS Digital RAP Teams group** (internal to NHS Digital - contact us @ [data.science@nhs.net](mailto:data.science@nhs.net))
    - [NHS Digital Github](https://github.com/NHSDigital)
    - Our very own [NHS Digital RAP Community of Practice](https://github.com/NHSDigital/rap-community-of-practice)

[1]: https://github.com/nhs-pycom/coding-club
